/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/ 
/*
  Copyright (C) 1998-1999 Johan Redestig
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
****************************************************************************/
#include "stdafx.h"
#include "TaskFolder.h"
#include "PopupMenu.h"
#include "PopupMaker.h"

#include "HeaderItem.h"
#include "TaskItem.h"
#include "SeparatorItem.h"

#include "Re5ources.h"

IgnoreList* tf_ignoreTag;

int TaskFolder::numIgnore = 0;
//int TaskFolder::TaskFolderFilterLevel = 3;
int TaskFolder::TaskFolderTimerInterval = 2000;
BOOL TaskFolder::sort = FALSE;
BOOL CALLBACK TaskFolderEnumWindowsProc(HWND hwnd, LPARAM lParam);
#define TASK_FOLDER_TIMER 4434

TaskFolder::TaskFolder(PopupMaker* pPopupMaker, LPSTR pszTitle, int FilterLevel) :
		FolderItem(NULL, pszTitle)
{
	m_pPopupMaker = pPopupMaker;
	m_nTimer = 0;
	TaskFolderFilterLevel = FilterLevel;

}
TaskFolder::TaskFolder(PopupMaker* pPopupMaker, LPSTR pszTitle) :
		FolderItem(NULL, pszTitle)
{
	m_pPopupMaker = pPopupMaker;
	m_nTimer = 0;
	TaskFolderFilterLevel = 3;
}

TaskFolder::~TaskFolder()
{
	if (m_pSubMenu)
		delete m_pSubMenu;
	m_pSubMenu = NULL;

	//if(m_nTimer) KillTimer(GetWindow
}

BOOL TaskFolder::Active(BOOL bActivate)
{
	if (bActivate && !m_bActive)
	{
		UpdateFolder();
	}

	return FolderItem::Active(bActivate);
}

void TaskFolder::AddWindow(HWND hWnd)
{
	if (IsAppWindowWithFilter( hWnd, TaskFolderFilterLevel ) )
	{
		MenuItem* pMenu = new TaskItem(hWnd);

		if ( (numIgnore > 0) && ( TaskFolderFilterLevel > 1 ) )
		{
			char title[1024];
			GetWindowText(hWnd, title, sizeof(title) );
			
			for (int i=0; i<numIgnore; ++i)
			{
				if (strstr(title, tf_ignoreTag[i].match))
					return;
			}
		}

		pMenu->SetHeight(m_pPopupMaker->m_nSubmenuHeight);
		pMenu->SetPainter(m_pPopupMaker->m_pEntry);
		pMenu->SetActivePainter(m_pPopupMaker->m_pSelEntry);

		if (m_pPopupMaker->m_bTaskIcons)
			pMenu->SetIcon(GetIconFromWindowWithTimeout(hWnd, FALSE, m_pPopupMaker->m_nGetIconTimeout));
 		m_pSubMenu->AddMenuItem(pMenu); 
	}
}

void TaskFolder::Attached(PopupMenu* pMenu)
{
	m_pParent = pMenu;
	m_pSubMenu = new PopupMenu(m_pPopupMaker->hInst);
	m_pParent->AddChild(m_pSubMenu);
	m_pSubMenu->SetParent(m_pParent);

	if (TaskFolderTimerInterval > 0)
		SetTimer(GetWindow(), TASK_FOLDER_TIMER, TaskFolderTimerInterval, NULL);
}

void TaskFolder::Timer(int nTimer)
{
	FolderItem::Timer(nTimer);

	if (TaskFolderTimerInterval > 0)
	{
		if (nTimer == TASK_FOLDER_TIMER && IsWindowVisible(m_pSubMenu->GetWindow()))
		{
			UpdateFolder();
			//		RedrawWindow(GetWindow(), NULL, NULL, RDW_INVALIDATE);
		}
	}
}

void TaskFolder::UpdateFolder()
{
	// _tremove the old items
	m_pSubMenu->DeleteMenuItems();

	// add a title
	m_pPopupMaker->AddHeaderItem(m_pSubMenu, GetTitle(), FALSE);

	// add the contents
	EnumWindows(TaskFolderEnumWindowsProc, (LPARAM)this);


	// add the footer
	m_pPopupMaker->AddBottomItem(m_pSubMenu);

	// fix the layout

	if (sort)
		m_pSubMenu->Sort();
	m_pSubMenu->Invalidate();
	m_pSubMenu->Validate();
}

/*void TaskFolder::ReadPopupTaskFolderIgnore()
{
	FILE * f = LCOpen(NULL);
	if (!f)
		return;
	unsigned int i;
	char * token[2];
	char buffer[4096];
	numIgnore = 0;

	for (i=0;i<2;++i)
		token[i] = new char[1024];
	while (LCReadNextConfig (f, "*PopupTasksFolderIgnore", buffer, sizeof(buffer)))
	{
		int count;
		count = LCTokenize (buffer, token, 2, NULL);
		if (!StripWS(token[1])) continue;
		if (count >= 2)
		{
			if (!tf_ignoreTag)
				tf_ignoreTag = (IgnoreList *)malloc(sizeof(IgnoreList));
			else
				tf_ignoreTag = (IgnoreList *)realloc(tf_ignoreTag, (numIgnore+1)*sizeof(IgnoreList));
			tf_ignoreTag[numIgnore].match = new char[strlen(token[1])+1];
			strcpy(tf_ignoreTag[numIgnore].match, token[1]);
			++numIgnore;
		}
	}
	LCClose(f);
	for (i = 0; i < 2; ++i)
		delete [] token[i];
	return;
}*/

BOOL CALLBACK TaskFolderEnumWindowsProc(HWND hwnd, LPARAM lParam)
{
	TaskFolder *pT = (TaskFolder*)lParam;
	pT->AddWindow(hwnd);

	return TRUE;
}

