/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/ 
/*
  Copyright (C) 1998-1999 Johan Redestig
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
****************************************************************************/
#include "stdafx.h"
#include "TaskItem.h"
#include "../lsapi/lsapi.h"
#include "Re5ources.h"
#include "PopupMaker.h"
#include "PopupMenu.h"

BOOL TaskItem::m_bWindowCaption = FALSE;
BOOL TaskItem::PopupStayOnSysMenu = FALSE;
FARPROC (__stdcall *SwitchToThisWindow)(HWND, int);

extern PopupMaker *pPopupMaker;

TaskItem::TaskItem(HWND hWnd)	: CommandItem(NULL, NULL, NULL)
{
	if ( 0 != GetWindowText(hWnd, pszWindowText, sizeof(pszWindowText)) )
		m_pszTitle = pszWindowText != NULL ? strdup(pszWindowText) : NULL ;
	else
		m_pszTitle = pPopupMaker->m_szTasksNoTitleString;

	m_hWnd = hWnd;

	SwitchToThisWindow = (FARPROC (__stdcall *)(HWND,
												int))GetProcAddress(GetModuleHandle("USER32.DLL"),
												"SwitchToThisWindow");
}

TaskItem::~TaskItem()
{
	// do not destroy the icon!
	m_hIcon = NULL;

	if (m_pszTitle)
		free(m_pszTitle);
	m_pszTitle = NULL;
}

void TaskItem::Invoke()
{
	m_pParent->Hide(HIDE_PARENTS);

	if (SwitchToThisWindow)
	{
		if (IsIconic(m_hWnd))
			SwitchToThisWindow(m_hWnd, TRUE);
		else
			if (!SendMessage(GetLitestepWnd(), LM_BRINGTOFRONT, 0, (LPARAM)m_hWnd))
				SwitchToThisWindow(m_hWnd, TRUE);
	}
	ShowWindow (m_hWnd, SW_SHOW); // in case it's hidden <g>
}

void TaskItem::RInvoke()
{
	ShowSystemMenu(m_hWnd);
}

BOOL TaskItem::Key(WPARAM nKey)
{
	switch (nKey)
	{
		case KILLSIGNAL:
			KillSelWindowProcess(m_hWnd);
			return FALSE;
		case CLOSESIGNAL:
			CloseSelWindow(m_hWnd);
			return FALSE;
		case HIDESIGNAL:
			HideSelWindow(m_hWnd);
			return FALSE;
		case SYSMENUSIGNAL:
			ShowSystemMenu(m_hWnd);
			return FALSE;
		default:
			return FALSE;
	}
}

void TaskItem::Mouse(int nMsg, int x, int y, WPARAM wParam)
{
	int modkey = GetModkeys(wParam);

	// CTRL+SHIFT key modifier
	if (0x00000011 == modkey)
	{
		if ( nMsg == WM_LBUTTONUP )
		{
			RECT r;
			POINT p;
			p.x = x;
			p.y = y;
			GetItemRect(&r);

			// KILL this mofo!
			if (PtInRect(&r, p))
				KillSelWindowProcess(m_hWnd);
		}
		else CommandItem::Mouse(nMsg, x, y, wParam);
	}

	// CTRL is modkey
	else if (0x00000001 == modkey)
	{
		if ( nMsg == WM_LBUTTONUP )
		{
			RECT r;
			POINT p;
			p.x = x;
			p.y = y;
			GetItemRect(&r);

			// hide this mofo!
			if (PtInRect(&r, p))
				HideSelWindow(m_hWnd);
		}
		else CommandItem::Mouse(nMsg, x, y, wParam);
	}

	// SHIFT key is modifier
	else if (0x00000010 == modkey)
	{
		if ( nMsg == WM_LBUTTONUP )
		{
			RECT r;
			POINT p;
			p.x = x;
			p.y = y;
			GetItemRect(&r);

			// close this mofo!
			if (PtInRect(&r, p))
				CloseSelWindow(m_hWnd);
		}
		else CommandItem::Mouse(nMsg, x, y, wParam);
	}
	else CommandItem::Mouse(nMsg, x, y, wParam);
}

void TaskItem::KillSelWindowProcess(HWND hWnd)
{
	Active(FALSE);
	DWORD ProcID;
	GetWindowThreadProcessId(hWnd,&ProcID);
	TerminateProcess(OpenProcess(PROCESS_TERMINATE, TRUE, ProcID), 0);
	Sleep(100);	/* gotta fix this, Sleep() is ugly */
	if ( 0 != GetWindowText(hWnd, pszWindowText, sizeof(pszWindowText)) )
		m_pszTitle = pszWindowText != NULL ? strdup(pszWindowText) : NULL ;
	else
		m_pszTitle = " [ PROCESS TERMINATED ] ";
	m_hIcon = NULL;
	Active(TRUE);
}

void TaskItem::HideSelWindow(HWND hWnd)
{
	if 	( !IsAppWindowWithFilter (hWnd, 12) ) 
	{
		SetWindowPos(	hWnd, NULL, 0, 0, 0, 0,
								SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW |
								SWP_NOACTIVATE | SWP_NOZORDER);
		if ( m_pszTitle = " [ WINDOW HIDDEN ] ")
		{
			Active(FALSE);
			if (pPopupMaker->m_bTaskIcons)
				SetIcon(GetIconFromWindowWithTimeout(hWnd, FALSE,pPopupMaker->m_nGetIconTimeout));
			if ( 0 != GetWindowText(hWnd, pszWindowText, sizeof(pszWindowText)) )
				m_pszTitle = pszWindowText != NULL ? strdup(pszWindowText) : NULL ;
			else
				m_pszTitle = pPopupMaker->m_szTasksNoTitleString;
			Active(TRUE);
		}
	}
	else
	{
		Active(FALSE);
		SetWindowPos(	hWnd, NULL, 0, 0, 0, 0,
								SWP_NOMOVE | SWP_NOSIZE | SWP_HIDEWINDOW |
								SWP_NOACTIVATE | SWP_NOZORDER);
		m_pszTitle = " [ WINDOW HIDDEN ] ";
		m_hIcon = NULL;
		Active(TRUE);
	}
}

void TaskItem::CloseSelWindow(HWND hWnd)
{
	Active(FALSE);
	PostMessage(m_hWnd,WM_CLOSE,0,0);
	Sleep(100);
	if ( 0 != GetWindowText(hWnd, pszWindowText, sizeof(pszWindowText)) )
		m_pszTitle = pszWindowText != NULL ? strdup(pszWindowText) : NULL ;
	else
		m_pszTitle = " [ WINDOW CLOSED ] ";
	m_hIcon = NULL;
	Active(TRUE);
}

void TaskItem::ShowSystemMenu(HWND hWnd)
{	
	if (PopupStayOnSysMenu)
	{
		HMENU systemMenu;
		systemMenu = GetSystemMenu( hWnd, FALSE );

		// This doesn't work at all for some windows.. IE5, GVIM,... (normalperson)
		POINT pt;
		GetCursorPos( &pt );

		int command = 0;

		command = TrackPopupMenu( systemMenu,
																	TPM_RETURNCMD | TPM_RIGHTBUTTON,
																	pt.x, pt.y, 0,
																	GetWindow(),
																	NULL );
		if ( 0 != command )
		{
			if ( ( command != SC_MINIMIZE && command != SC_CLOSE ) )
				SetForegroundWindow( hWnd );
			PostMessage( hWnd, WM_SYSCOMMAND, (WPARAM) command, MAKELPARAM( pt.x, pt.y ) );
		}
	}
	else if (!PopupStayOnSysMenu)
	{
		SetForegroundWindow(GetLastActivePopup(hWnd));
		PostMessage(hWnd, 0x313, 0, GetMessagePos() );
	}
}


void TaskItem::Paint(HDC hDC)
{
	if (m_bWindowCaption)
	{
		RECT r;
		GetItemRect(&r);

		if (m_bActive)
			DrawCaption(m_hWnd, hDC, &r, DC_ACTIVE | DC_ICON | DC_TEXT);
		else
			DrawCaption(m_hWnd, hDC, &r, DC_ICON | DC_TEXT);

		if (m_bDrawBevel)
			PaintBevel(hDC, m_nTop);
	}
	else
	{
		CommandItem::Paint(hDC);
	}

}


